// C Source File
// Created 3/9/2006; 7:42:31 AM

//make sure a value is returned
#define RETURN_VALUE

//compile for all calculators, although i only
//have a ti89 titanium so it may not work on the 
//others.
#define USE_TI89
#define USE_TI92PLUS
#define USE_V200

//this is required for some argument functions
#define MIN_AMS 101

//obviously required for all calculator functions
#include <tigcclib.h>

//checks a bit within an integer. Returns zero if 
//bit is not set or non-zero otherwise.
int check_bit(int num, int bit)
{
  //used for removing all other bits in the number
  //set to 00000001
  int mask; mask = 1;

  //move the one by the amount specified in bit
  //if bit = 3 then mask now equals 00001000
  mask <<= bit;

  //return num&mask, this &'s each of the bits
  //in num to its equivalent bit in mask. Because
  //mask only has one bit set, the most number of bits
  //returned will also be one.
  return num & mask;
}

//sigma function. takes the number of terms, the numbers required
//and the size of the array of numbers and returns the function.
//the number of terms is the number of variables, if num_terms is 4
//then a,b,c,d are used. If the array sigma = {0,1,2,3,4,...,x} then
//the return values should be a string containing {m0,m1,m2,m3,m4,...,mx}
char * sigmaf(int num_terms, int * sigma, int sigma_size)
{
  //the number of minterms = 2^(terms)
  int num_minterms; num_minterms = pow(2,num_terms);

  //the function string, this is returned at the end
  //it is set to the overly large value to avoid using
  //extra memory handles. 
  char * func; func=malloc(3*num_terms*sigma_size);

  //the next character in the function
  int next; next = 0;

  //the next number in the sigma array
  int sigma_next;

   //loop through the array of numbers (for each minterm required)
  for (sigma_next = 0; sigma_next < sigma_size; sigma_next++)
    {
      //loop variable x
      int x;

      //loop through the individual terms within a minterm
      for (x = 0; x < num_terms; x++)
	{
	  //check to see if the term should be inverted
	  //this is done by checking the binary bit of the number
	  //used to count (x)
	  if (check_bit(sigma[sigma_next],num_terms -  x - 1) == 0)
	    {
	      //invert the variable by putting a dash in front of it
	      //this looks messy but is for compatibility with 'logic'
	      //another program available at "www.ticalc.org" 
	      //'logic' is the program that does the actual solving
	      func[next++] = '-';
	    }
	  //put the variable in the function string. 
	  // the 26 variable limit created
	  //should be quite easily enough (other limitations may stop
	  //26 variables being allowed anyway) 
	  //if more than 26 is attempted, strange symbols may appear
	  //(not recommended)
	  func[next++] = 'a'+x;

	  //add the 'times'/'and' operator
	  //if this is the last element it will be overwritten
	  //by the 'plus'/'or' operator.
	  func[next++] = '*';
	}
      //add the 'plus'/'or' operator
      //if this is the last minterm, it will be overwritten
      //by the terminating character
      func[next-1] = '+';
    }
  //add terminating character
  func[next-1] = '\0';

  //return the function string
  return func;
}

// Main Function
void _main(void)
{
  //make a value string to return.
  //start with it set to an error message and
  //then replace it with the data if everything runs smoothly
  char * value; value = "Usage: minterms(num_terms, sigma_size, sigma)";

  //holder for the number of terms
  int num_terms;

  //holder for the number of minterms required
  int sigma_size;

  //loop variable
  int x;
  
  //check for a positive integer if not return
  if (GetArgType(top_estack) != POSINT_TAG)
    return;

  //collect the argument from top_estack
  //note - it says in the tigcc docs that top_estack should
  //not be used with this function, but since otherwise it would
  //have to be cleaned up anyway, it doesnt make much difference
  num_terms = GetIntArg(top_estack);
	
  //check for another positive integer, if not return
  if (GetArgType(top_estack) != POSINT_TAG)
    return;

  //collect the argument from top_estack
  sigma_size = GetIntArg(top_estack);
	
  //allocate memory for sigma
  int * sigma; sigma = malloc(sigma_size * sizeof(int));
 
  //loop through sigma
  for (x = 0; x < sigma_size; x++)
    {
      //check for a positive integer, if not return
      if (GetArgType(top_estack) != POSINT_TAG)
	return;

      //collect each integer argument into an element of sigma
      sigma[x] = GetIntArg(top_estack);
    }
	
  //call the sigma function with the correct arguments
  value = sigmaf(num_terms,sigma,sigma_size);
	
  //push the value string, this returns it
  push_zstr(value);
}
